#!/usr/bin/env node

const fsp = require("fs/promises");
const fs = require("fs");
const path = require("path");
const { default: inquirer } = require("inquirer");

async function main() {
    const replacements = [
        // Prefix css variables
        { from: "--font-size(?!-)", to: "--hy-font-size" },
        { from: "--font-size-small", to: "--hy-font-size-small" },
        { from: "--font-family", to: "--hy-font-family" },

        { from: "--flex-gap", to: "--hy-flex-gap" },
        { from: "--section-padding", to: "--hy-section-padding" },
        { from: "--section-border-radius", to: "--hy-section-border-radius" },
        { from: "--section-shadow", to: "--hy-section-shadow" },
        { from: "--section-divider", to: "--hy-section-divider" },

        { from: "--element-border-secondary", to: "--hy-element-border-secondary" },
        { from: "--element-border-accent", to: "--hy-element-border-accent" },
        { from: "--element-border-error", to: "--hy-element-border-error" },

        { from: "--element-margin-value", to: "--hy-element-margin-value" },
        { from: "--element-margin(?!-)", to: "--hy-element-margin" },
        { from: "--element-padding(?!-)", to: "--hy-element-padding" },
        { from: "--element-padding-light", to: "--hy-element-padding-md" },
        { from: "--element-padding-thin", to: "--hy-element-padding-sm" },

        { from: "--element-transition", to: "--hy-element-transition" },
        { from: "--tabbar-transition", to: "--hy-tabbar-transition" },
        { from: "--element-shadow", to: "--hy-element-shadow" },
        { from: "--element-border-radius", to: "--hy-element-border-radius" },

        { from: "--popover-shadow", to: "--hy-popover-shadow" },
        { from: "--dropdown-top-shadow", to: "--hy-dropdown-top-shadow" },

        { from: "--tabbar-shadow", to: "--hy-tabbar-shadow" },
        { from: "--sidebar-shadow", to: "--hy-sidebar-shadow" },
        { from: "--header-shadow-cover", to: "--hy-header-shadow-cover" },

        { from: "--font-color-white", to: "--hy-font-color-white" },
        { from: "--font-color-black", to: "--hy-font-color-black" },

        { from: "--font-color-contrast", to: "--hy-font-color-contrast" },
        { from: "--font-color-match", to: "--hy-font-color-match" },

        { from: "--font-color-dark", to: "--hy-font-color-dark" },
        { from: "--font-color-bright", to: "--hy-font-color-bright" },

        { from: "--body-bg-color", to: "--hy-body-bg-color" },
        { from: "--accent-color", to: "--hy-accent-color" },
        { from: "--loader-ring-bg-color", to: "--hy-loader-ring-bg-color" },

        { from: "--color-blue", to: "--hy-color-blue" },
        { from: "--color-pink", to: "--hy-color-pink" },
        { from: "--color-yellow", to: "--hy-color-yellow" },
        { from: "--color-green", to: "--hy-color-green" },
        { from: "--color-red", to: "--hy-color-red" },
        { from: "--color-light-blue", to: "--hy-color-light-blue" },
        { from: "--color-purple", to: "--hy-color-purple" },
        { from: "--color-gray-font", to: "--hy-font-color-gray" },
        { from: "--color-gray-1", to: "--hy-color-gray-1" },
        { from: "--color-gray-2", to: "--hy-color-gray-2" },
        { from: "--color-gray-3", to: "--hy-color-gray-3" },

        { from: "--section-shadow-color", to: "--hy-section-shadow-color" },
        { from: "--section-bg-color", to: "--hy-section-bg-color" },

        { from: "--element-primary-bg-color", to: "--hy-element-primary-bg-color" },
        { from: "--element-primary-hover-bg-color", to: "--hy-element-primary-hover-bg-color" },

        { from: "--element-secondary-bg-color", to: "--hy-element-secondary-bg-color" },
        { from: "--element-secondary-hover-bg-color", to: "--hy-element-secondary-hover-bg-color" },
        { from: "--element-border-secondary-color", to: "--hy-element-border-secondary-color" },

        { from: "--element-shadow-color", to: "--hy-element-shadow-color" },
        { from: "--section-divider-color", to: "--hy-section-divider-color" },
        { from: "--gray-tabbar-bg-color", to: "--hy-gray-tabbar-bg-color" },
        { from: "--gray-tabbar-ghost-color", to: "--hy-gray-tabbar-ghost-color" },

        { from: "--button-transparent-bg", to: "--hy-button-transparent-bg" },
        { from: "--button-transparent-hover", to: "--hy-button-transparent-hover" },

        { from: "--button-light-blue-bg", to: "--hy-button-light-blue-bg" },
        { from: "--button-light-blue-hover", to: "--hy-button-light-blue-hover" },
        { from: "--button-light-blue-font", to: "--hy-button-light-blue-font" },

        { from: "--button-light-red-bg", to: "--hy-button-light-red-bg" },
        { from: "--button-light-red-hover", to: "--hy-button-light-red-hover" },
        { from: "--button-light-red-font", to: "--hy-button-light-red-font" },

        { from: "--button-light-orange-bg", to: "--hy-button-light-orange-bg" },
        { from: "--button-light-orange-hover", to: "--hy-button-light-orange-hover" },
        { from: "--button-light-orange-font", to: "--hy-button-light-orange-font" },

        { from: "--button-light-green-bg", to: "--hy-button-light-green-bg" },
        { from: "--button-light-green-hover", to: "--hy-button-light-green-hover" },
        { from: "--button-light-green-font", to: "--hy-button-light-green-font" },

        { from: "--modal-bg-color", to: "--hy-modal-bg-color" },
        { from: "--modal-cover-color-mobile", to: "--hy-modal-cover-color-mobile" },
        { from: "--modal-cover-color-desktop", to: "--hy-modal-cover-color-desktop" },
        { from: "--modal-shadow-color", to: "--hy-modal-shadow-color" },

        { from: "--popover-shadow-color", to: "--hy-popover-shadow-color" },
        { from: "--popover-bg-color-match", to: "--hy-popover-bg-color-match" },
        { from: "--popover-bg-color-contrast", to: "--hy-popover-bg-color-contrast" },
        { from: "--popover-border-color-match", to: "--hy-popover-border-color-match" },
        { from: "--popover-border-color-contrast", to: "--hy-popover-border-color-contrast" },

        { from: "--tabbar-shadow-color", to: "--hy-tabbar-shadow-color" },
        { from: "--sidebar-shadow-color", to: "--hy-sidebar-shadow-color" },

        // REPLACE element variant variable names
        { from: "--hy-element-primary-bg-color", to: "--hy-variant-primary-bg-color" },
        { from: "--hy-element-primary-hover-bg-color", to: "--hy-variant-primary-hover-color" },

        { from: "--hy-element-secondary-bg-color", to: "--hy-variant-secondary-bg-color" },
        { from: "--hy-element-secondary-hover-bg-color", to: "--hy-variant-secondary-hover-color" },

        { from: "--hy-button-transparent-bg", to: "--hy-variant-ghost-bg-color" },
        { from: "--hy-button-transparent-hover", to: "--hy-variant-ghost-hover-color" },

        { from: "--hy-button-light-blue-bg", to: "--hy-variant-ghost-blue-bg-color" },
        { from: "--hy-button-light-blue-hover", to: "--hy-variant-ghost-blue-hover-color" },
        { from: "--hy-button-light-blue-font", to: "--hy-variant-ghost-blue-font-color" },

        { from: "--hy-button-light-red-bg", to: "--hy-variant-ghost-red-bg-color" },
        { from: "--hy-button-light-red-hover", to: "--hy-variant-ghost-red-hover-color" },
        { from: "--hy-button-light-red-font", to: "--hy-variant-ghost-red-font-color" },

        { from: "--hy-button-light-orange-bg", to: "--hy-variant-ghost-yellow-bg-color" },
        { from: "--hy-button-light-orange-hover", to: "--hy-variant-ghost-yellow-hover-color" },
        { from: "--hy-button-light-orange-font", to: "--hy-variant-ghost-yellow-font-color" },

        { from: "--hy-button-light-green-bg", to: "--hy-variant-ghost-green-bg-color" },
        { from: "--hy-button-light-green-hover", to: "--hy-variant-ghost-green-hover-color" },
        { from: "--hy-button-light-green-font", to: "--hy-variant-ghost-green-font-color" },

        // REPLACE changed variable names
        { from: "--hy-color-gray-font", to: "--hy-font-color-gray" },
        { from: "--hy-element-padding-light", to: "--hy-element-padding-md" },
        { from: "--hy-element-padding-thin", to: "--hy-element-padding-sm" },

        // PROPS: type -> variant
        {
            from: /(<hy-button\b[^>]*?)(\s+(type|:type|v-bind:type)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("type", "variant"),
        },
        {
            from: /(<hy-popover\b[^>]*?)(\s+(type|:type|v-bind:type)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("type", "variant"),
        },
        {
            from: /(<hy-tooltip\b[^>]*?)(\s+(type|:type|v-bind:type)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("type", "variant"),
        },
        {
            from: /(<hy-smart-menu\b[^>]*?)(\s+(type|:type|v-bind:type)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("type", "variant"),
        },
        {
            from: /(<hy-tabbar\b[^>]*?)(\s+(type|:type|v-bind:type)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("type", "variant"),
        },

        // PROPS: border -> showBorder
        {
            from: /(<hy-dropdown\b[^>]*?)(\s+(border|:border|v-bind:border)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("border", "showBorder"),
        },
        {
            from: /(<hy-section\b[^>]*?)(\s+(border|:border|v-bind:border)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("border", "showBorder"),
        },
        {
            from: /(<hy-select-dropdown\b[^>]*?)(\s+(border|:border|v-bind:border)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("border", "showBorder"),
        },
        {
            from: /(<hy-table\b[^>]*?)(\s+(border|:border|v-bind:border)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("border", "showBorder"),
        },

        // PROPS: extendHeadline -> doExtendHeadline
        {
            from: /(<hy-header\b[^>]*?)(\s+(extendHeadline|:extendHeadline|v-bind:extendHeadline)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("extendHeadline", "doExtendHeadline"),
        },

        // PROPS: hover -> doOpenOnHover
        {
            from: /(<hy-popover\b[^>]*?)(\s+(hover|:hover|v-bind:hover)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("hover", "doOpenOnHover"),
        },

        // PROPS: dividers -> showDividers
        {
            from: /(<hy-section\b[^>]*?)(\s+(dividers|:dividers|v-bind:dividers)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("dividers", "showDividers"),
        },

        // PROPS: enableSearch -> showSearch
        {
            from: /(<hy-select-dropdown\b[^>]*?)(\s+(enableSearch|:enableSearch|v-bind:enableSearch)\s*=\s*["'].*?["'])([^>]*?>)/gs,
            to: replaceProp("enableSearch", "showSearch"),
        },

        // VARIANTS: light-blue to ghost-blue
        {
            from: /((?::|v-bind:)?variant\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])light-blue(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("ghost-blue"),
        },

        // VARIANTS: light-bg-blue to ghost-bg-blue
        {
            from: /((?::|v-bind:)?variant\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])light-bg-blue(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("ghost-bg-blue"),
        },

        // VARIANTS: light-red to ghost-red
        {
            from: /((?::|v-bind:)?variant\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])light-red(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("ghost-red"),
        },

        // VARIANTS: light-bg-red to ghost-bg-red
        {
            from: /((?::|v-bind:)?variant\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])light-bg-red(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("ghost-bg-red"),
        },

        // VARIANTS: light-orange to ghost-yellow
        {
            from: /((?::|v-bind:)?variant\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])light-orange(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("ghost-yellow"),
        },

        // VARIANTS: light-bg-orange to ghost-bg-yellow
        {
            from: /((?::|v-bind:)?variant\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])light-bg-orange(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("ghost-bg-yellow"),
        },

        // VARIANTS: light-green to ghost-green
        {
            from: /((?::|v-bind:)?variant\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])light-green(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("ghost-green"),
        },

        // VARIANTS: light-bg-green to ghost-bg-green
        {
            from: /((?::|v-bind:)?variant\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])light-bg-green(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("ghost-bg-green"),
        },

        // VARIANTS: transparent to ghost
        {
            from: /((?::|v-bind:)?variant\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])transparent(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("ghost"),
        },

        // VARIANTS: transparent-bg to ghost-bg
        {
            from: /((?::|v-bind:)?variant\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])transparent-bg(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("ghost-bg"),
        },

        // SIZES: normal to lg
        {
            from: /((?::|v-bind:)?size\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])normal(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("lg"),
        },

        // SIZES: light to md
        {
            from: /((?::|v-bind:)?size\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])light(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("md"),
        },

        // SIZES: thin to sm
        {
            from: /((?::|v-bind:)?size\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])thin(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("sm"),
        },

        // TABBAR VARIANT: blue to primary
        {
            from: /(<hy-tabbar[^>]*?(?::|v-bind:)?variant\s*=\s*["'](?:[^"']*?)|\?\s*["'](?:[^"']*?)["']\s*:\s*["'])blue(["']|(?:[^"']*?)["'])/g,
            to: replacePropValue("primary"),
        },
    ];

    let args = await getArguments();

    await replaceAllInstancesInDirectory(replacements, args.directory);

    console.log("\nUpgrade completed! Please review the changes and test the application.");
}

async function replaceAllInstancesInDirectory(replacements, inputPath) {
    // Validate inputs
    if (!Array.isArray(replacements) || !inputPath || !fs.existsSync(inputPath)) {
        throw new Error("Invalid arguments");
    }

    // Function to replace all instances in a file
    async function replaceAllInstancesInFile(filePath) {
        // Only parse code files
        let allowedExtentions = [".html", ".vue", ".js", ".css", ".scss", ".sass", ".less"];
        let ext = path.extname(filePath);
        if (!allowedExtentions.includes(ext)) return;

        try {
            // Read file content
            let content = await fsp.readFile(filePath, "utf8");

            // Apply each replacement
            replacements.forEach(({ from, to }) => {
                let regex = new RegExp(from, "g");

                if (typeof to == "function") content = to(content, { regex });
                else content = content.replace(regex, to);
            });

            // Write back changes
            await fsp.writeFile(filePath, content, "utf8");
            console.log(`Processed: ${filePath}`);
        } catch (error) {
            console.error(`Error processing ${filePath}:`, error);
        }
    }

    // Function to scan a directory
    async function scanDirectory(dir) {
        const entries = await fsp.readdir(dir);

        for (const entry of entries) {
            const fullPath = path.join(dir, entry);

            // Check if the entry is a directory
            const stats = await fsp.stat(fullPath);

            if (stats.isDirectory()) {
                await scanDirectory(fullPath);
            } else {
                await replaceAllInstancesInFile(fullPath);
            }
        }
    }

    // Execute the replacement for the given directory
    const stats = await fsp.stat(inputPath);

    if (stats.isDirectory()) {
        await scanDirectory(inputPath);
    } else {
        await replaceAllInstancesInFile(inputPath);
    }
}

async function getArguments() {
    return await inquirer.prompt([
        {
            type: "input",
            name: "directory",
            message: "Directory or file to upgrade: ",
            default: path.join(process.cwd(), "src"),
            validate: (input) => fs.existsSync(input),
        },
    ]);
}

main();

// Helper functions
function replaceProp(from, to) {
    return (htmlString, { regex }) => {
        return htmlString.replace(regex, (match, prefix, propAttr, attrName, suffix) => {
            // Replace 'from' with 'to' but keep the rest of the attribute intact
            let newAttr;
            if (attrName === from) {
                newAttr = propAttr.replace(new RegExp(`\\b${from}\\b`), to);
            } else if (attrName === `:${from}`) {
                newAttr = propAttr.replace(new RegExp(`:${from}`), `:${to}`);
            } else if (attrName === `v-bind:${from}`) {
                newAttr = propAttr.replace(new RegExp(`v-bind:${from}`), `v-bind:${to}`);
            }
            return prefix + newAttr + suffix;
        });
    };
}

function replacePropValue(to) {
    return (htmlContent, { regex }) => {
        return htmlContent.replace(regex, (match, prefix, suffix) => {
            return prefix + to + suffix;
        });
    };
}
