import * as components from "./components.js";
import GlobalBSLManager from "./utils/GlobalBSLManager.js";

import "./globals.scss";
import useLightmode from "./themes/useLightmode.scss?inline";
import useDarkmode from "./themes/useDarkmode.scss?inline";
import useSystem from "./themes/useSystem.scss?inline";

import * as brandThemeUtils from "./utils/brandThemeUtils.js";
import brandTheme_classic from "./brandThemes/classic.js";

export default function install(app, options = {}) {
    app.config.globalProperties.hy_defaults = {};
    app.config.globalProperties.hy_bsl_manager = new GlobalBSLManager();

    // Output config, if in dev mode
    if (import.meta.env.DEV) {
        console.log("HyperUI config: ", options);
    }

    // Apply light/darkmode theme
    let themes = {
        system: useSystem,
        dark: useDarkmode,
        light: useLightmode,
    };

    let themeElement = document.createElement("style");
    themeElement.id = "hy-theme-inject";
    themeElement.innerHTML = themes[options?.theme ?? "system"];
    document.body.appendChild(themeElement);

    // Apply brand theme
    let compiledBrandTheme = brandThemeUtils.compileBrandTheme(options?.brandTheme ?? brandTheme_classic);
    let brandThemeVariables = brandThemeUtils.generateCssVariables(compiledBrandTheme);
    let brandThemeUtilityClasses = brandThemeUtils.generateCssUtilityClasses(compiledBrandTheme);
    brandThemeUtils.applyCssVariables(brandThemeVariables);
    brandThemeUtils.applyCssUtilityClasses(brandThemeUtilityClasses);
    app.config.globalProperties.hy_brand_theme = compiledBrandTheme;

    // Save selected default size option
    app.config.globalProperties.hy_defaults.size = options?.size ?? "lg";
    app.config.globalProperties.hy_defaults.variant = options?.variant ?? "secondary";

    // Setup modals and popover container
    ["hy-modals-container", "hy-popovers-container"].forEach((name) => {
        if (document.getElementById(name) != null) return;

        let el = document.createElement("div");
        el.id = name;
        document.body.appendChild(el);
    });

    // Install components
    Object.keys(components).forEach((componentName) => {
        app.component(componentName, components[componentName]);
    });

    // Add resize observer to address 100vw overflow issue if classic scrollbars are shown
    new ResizeObserver(() => {
        let vw = document.documentElement.clientWidth / 100;
        document.documentElement.style.setProperty("--hy-vw", `${vw}px`);
    }).observe(document.documentElement);
}
