import { disableBodyScroll, enableBodyScroll } from "body-scroll-lock";
import { computed, watch, ref } from "vue";

export default class GlobalBSLManager {
    constructor() {
        this.lockedElements = ref([]);

        // Last element
        this.lastElement = computed(() => this.lockedElements.value[this.lockedElements.value.length - 1]);

        // Lock/Unlock the last element in the list, whenever it changes
        watch(
            () => this.lastElement.value,
            (newLastElement, oldLastElement) => {
                if (oldLastElement) enableBodyScroll(oldLastElement); // Unlock for old last element

                if (newLastElement) disableBodyScroll(newLastElement); // Lock for new last element
            },
            { deep: true },
        );
    }

    push(element) {
        this.lockedElements.value.push(element);
    }

    remove(element) {
        const index = this.lockedElements.value.indexOf(element);

        if (index > -1) this.lockedElements.value.splice(index, 1);
    }
}
